#include "mta_interface.h"

#include <stdio.h>
#include <sys/wait.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <assert.h>

static const char exim_mailq_cmd[]="/usr/bin/mailq";
static const char exim_queue_directory[]="/var/spool/exim/input";

static const char sendmail_mailq_cmd[]="/usr/bin/mailq";
static const char sendmail_queue_directory[]="/var/spool/mqueue";

static const char* mta_mailq_cmd;
static const char* mta_queue_directory;

static int directory_exists(const char* dir)
{
  struct stat s;
  if(stat(dir,&s)==0){
    return 1;
  }else{
    return 0;
  }
}

static int detect_exim(void)
{
  return directory_exists(exim_queue_directory);
}

static int detect_sendmail(void)
{
  return directory_exists(sendmail_queue_directory);
}

void init_mta_interface(void)
{
  int exim_detected;
  int sendmail_detected;
  exim_detected=detect_exim();
  sendmail_detected=detect_sendmail();
  assert(!(exim_detected&&sendmail_detected));
  assert(exim_detected||sendmail_detected);
  if(exim_detected){
    //fprintf(stderr,"Exim detected\n");
    mta_mailq_cmd=exim_mailq_cmd;
    mta_queue_directory=exim_queue_directory;
  }else if(sendmail_detected){
    //fprintf(stderr,"Sendmail detected\n");
    mta_mailq_cmd=sendmail_mailq_cmd;
    mta_queue_directory=sendmail_queue_directory;
  }
}

static int has_queue_directory_changed(void)
{
  static time_t previous_mtime=0;
  struct stat s;

  assert(stat(mta_queue_directory,&s)==0);
  if(s.st_mtime==previous_mtime){
    return 0;
  }else{
    previous_mtime=s.st_mtime;
    return 1;
  }
}

int get_mail_queue_length(void)
{
  FILE* mta_cmd_output_pipe;
  const int read_buf_sz=256;
  char read_buf[read_buf_sz];
  int queue_length;
  int status;
  static int previous_mail_queue_length;

  if(!has_queue_directory_changed()){
    return previous_mail_queue_length;
  }
  mta_cmd_output_pipe=popen(mta_mailq_cmd,"r");
  assert(mta_cmd_output_pipe);
  queue_length=0;
  while(fgets(read_buf,read_buf_sz,mta_cmd_output_pipe)){
    if(strncmp(read_buf,"          ",10)==0){
      queue_length++;
    }
  }
  status=pclose(mta_cmd_output_pipe);
  if(WIFEXITED(status)){
    if(WEXITSTATUS(status)!=0){
      return -1;
    }else{
      previous_mail_queue_length=queue_length;
      return queue_length;
    }
  }else{
    return -1;
  }
}


